'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var babel = _interopDefault(require('@babel/core'));
var fs = _interopDefault(require('fse'));
var path = _interopDefault(require('path'));
var pHTML = _interopDefault(require('phtml'));
var postcss = _interopDefault(require('postcss'));

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }

  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}

function _asyncToGenerator(fn) {
  return function () {
    var self = this,
        args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);

      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
      }

      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
      }

      _next(undefined);
    });
  };
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);

    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
        return Object.getOwnPropertyDescriptor(source, sym).enumerable;
      }));
    }

    ownKeys.forEach(function (key) {
      _defineProperty(target, key, source[key]);
    });
  }

  return target;
}

function expressVariable(dir, rawopts) {
  return (
    /*#__PURE__*/
    function () {
      var _ref = _asyncToGenerator(function* (request, response, next) {
        // transform GET requests, ignore the rest
        const isValidRequestMethod = /^GET$/i.test(request.method);

        if (!isValidRequestMethod) {
          return next();
        } // configure the JS and CSS options


        const cssOpts = Object(Object(rawopts).css);
        const htmlOpts = Object(Object(rawopts).html);
        const jsOpts = Object(Object(rawopts).js);
        const index = 'index' in Object(rawopts) ? rawopts.index ? String(rawopts.index) : false : 'index.html';
        const opts = {
          css: _objectSpread({}, cssOpts, {
            fileExtensions: [].concat(cssOpts.fileExtensions || ['css', 'pcss'])
          }),
          html: _objectSpread({}, htmlOpts, {
            fileExtensions: [].concat(htmlOpts.fileExtensions || ['html', 'phtml'])
          }),
          js: _objectSpread({}, jsOpts, {
            fileExtensions: [].concat(jsOpts.fileExtensions || ['js', 'mjs'])
          })
        }; // determine path information about the request

        let fullpath = path.resolve(`${dir || ''}${request.path}`);
        const extension = path.extname(fullpath).slice(1); // determine whether a JS or CSS request is being made, ignore the rest

        const isCssExtension = opts.css.fileExtensions.includes(extension);
        const isJsExtension = opts.js.fileExtensions.includes(extension);
        let isHtmlExtension = opts.html.fileExtensions.includes(extension);

        if (!isJsExtension && !isCssExtension && !isHtmlExtension) {
          try {
            const indexpath = path.resolve(fullpath, index);
            yield fs.stat(indexpath);
            fullpath = indexpath;
            isHtmlExtension = true;
          } catch (error) {
            return next();
          }
        }

        try {
          // determine if the file exists and has been modified since the last request
          const stat = yield fs.stat(fullpath);
          const lastModifiedUTCString = new Date(stat.mtimeMs).toUTCString();
          const isTheFileUnmodified = request.headers['if-modified-since'] === lastModifiedUTCString;
          let buffer;

          if (isTheFileUnmodified) {
            // return a 304 response header if the file has not been modified since the last request
            response.writeHead(304);
            return response.end();
          } else {
            // read the source as UTF-8 content
            const source = yield fs.readFile(fullpath, 'utf8');

            if (isCssExtension) {
              // configure postcss process options
              const processOpts = _objectSpread({}, opts.css, {
                from: fullpath
              });

              delete processOpts.fileExtensions;
              delete processOpts.plugins;
              const plugins = Array.from(opts.css.plugins || []); // process the source using postcss

              const result = yield postcss(plugins).process(source, processOpts);
              buffer = result.css;
            } else if (isHtmlExtension) {
              // configure phtml process options
              const processOpts = _objectSpread({}, opts.html, {
                from: fullpath
              });

              delete processOpts.fileExtensions;
              delete processOpts.index;
              delete processOpts.plugins;
              const plugins = Array.from(opts.html.plugins || []); // process the source using phtml

              const result = yield pHTML.use(plugins).process(source, processOpts);
              buffer = result.html;
            } else if (isJsExtension) {
              // configure babel transform options
              const transformOpts = _objectSpread({}, opts.js, {
                filename: fullpath
              });

              delete transformOpts.fileExtensions; // process the source using babel

              const result = yield babel.transformAsync(source, transformOpts);
              buffer = result.code;
            }
          } // determine the content type being served


          const contentType = `${isHtmlExtension ? 'text/html' : isJsExtension ? 'application/javascript' : 'text/css'}; charset=UTF-8`; // serve the appropriate response headers for the request

          response.setHeader('Content-Length', buffer.length);
          response.setHeader('Content-Type', contentType);
          response.setHeader('Last-Modified', lastModifiedUTCString); // serve the appropriate response for the request

          response.write(buffer); // end the response

          return response.end();
        } catch (error) {
          // return the forwarded error
          return next(error);
        }
      });

      return function (_x, _x2, _x3) {
        return _ref.apply(this, arguments);
      };
    }()
  );
}

module.exports = expressVariable;
//# sourceMappingURL=index.js.map
